/*
 * Copyright (c) 2018, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * WSO2 Inc. licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.wso2.carbon.identity.application.authentication.framework.config.model.graph.js;

import org.wso2.carbon.identity.application.authentication.framework.util.FrameworkConstants;

import javax.servlet.http.HttpServletResponse;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

/**
 * Javascript wrapper for Java level HttpServletResponse.
 * This provides controlled access to HttpServletResponse object via provided javascript native syntax.
 * e.g
 * response.headers.["Set-Cookie"] = ['crsftoken=xxxxxssometokenxxxxx']
 * <p>
 * instead of
 * context.getResponse().addCookie(cookie);
 * <p>
 * Also it prevents writing an arbitrary values to the respective fields, keeping consistency on runtime HttpServletResponse.
 */
public class JsServletResponse extends AbstractJSObjectWrapper<HttpServletResponse> {

    public JsServletResponse(HttpServletResponse wrapped) {

        this.wrapped = wrapped;
    }

    @Override
    public Object getMember(String name) {

        if (wrapped == null) {
            return super.getMember(name);
        }

        switch (name) {
            case FrameworkConstants.JSAttributes.JS_HEADERS:
                Map headers = new HashMap();
                Collection<String> headerNames = wrapped.getHeaderNames();
                if (headerNames != null) {
                    for (String element : headerNames) {
                        headers.put(element, wrapped.getHeader(element));
                    }
                }
                return new JsHeaders(headers, wrapped);
            default:
                return super.getMember(name);
        }
    }

    @Override
    public boolean hasMember(String name) {

        if (wrapped == null) {
            return false;
        }
        switch (name) {
            case FrameworkConstants.JSAttributes.JS_HEADERS:
                return wrapped.getHeaderNames() != null;
            default:
                return super.hasMember(name);
        }
    }
}
